Now that we've taken isr_el1 out of the box, there are a few more places
we could use it. During __guest_exit() we need to consume any SError left
pending by the guest so it doesn't contaminate the host. With v8.2 we use
the ESB-instruction. For systems without v8.2, we use dsb+isb and unmask
SError. We do this on every guest exit.

Use the same dsb+isr_el1 trick, this lets us know if an SError is pending
after the dsb, allowing us to skip the isb and self-synchronising PSTATE
write if its not.

This means SError remains masked during KVM's world-switch, so any SError
that occurs during this time is reported by the host, instead of causing
a hyp-panic.

As we're benchmarking this code lets polish the layout. If you give gcc
likely()/unlikely() hints in an if() condition, it shuffles the generated
assembly so that the likely case is immediately after the branch. Lets
do the same here.

Signed-off-by: James Morse <james.mo...@arm.com>
---
Tested on A57 with v5.2-rc1, do_hvc from [0]
v5.2-rc1            mean:4339 stddev:33
v5.2-rc1+patches1-6 mean:4309 stddev:26
with series 0.69% faster
[0] https://git.kernel.org/pub/scm/linux/kernel/git/maz/kvm-ws-tests.git/

 arch/arm64/kvm/hyp/entry.S | 14 ++++++++++----
 1 file changed, 10 insertions(+), 4 deletions(-)

diff --git a/arch/arm64/kvm/hyp/entry.S b/arch/arm64/kvm/hyp/entry.S
index a5a4254314a1..c2de1a1faaf4 100644
--- a/arch/arm64/kvm/hyp/entry.S
+++ b/arch/arm64/kvm/hyp/entry.S
@@ -161,18 +161,24 @@ alternative_if ARM64_HAS_RAS_EXTN
        orr     x0, x0, #(1<<ARM_EXIT_WITH_SERROR_BIT)
 1:     ret
 alternative_else
-       // If we have a pending asynchronous abort, now is the
-       // time to find out. From your VAXorcist book, page 666:
+       dsb     sy              // Synchronize against in-flight ld/st
+       mrs     x2, isr_el1
+       and     x2, x2, #(1<<8) // ISR_EL1.A
+       cbnz    x2, 2f
+       ret
+
+2:
+       // We know we have a pending asynchronous abort, now is the
+       // time to flush it out. From your VAXorcist book, page 666:
        // "Threaten me not, oh Evil one!  For I speak with
        // the power of DEC, and I command thee to show thyself!"
        mrs     x2, elr_el2
+alternative_endif
        mrs     x3, esr_el2
        mrs     x4, spsr_el2
        mov     x5, x0
 
-       dsb     sy              // Synchronize against in-flight ld/st
        msr     daifclr, #4     // Unmask aborts
-alternative_endif
 
        // This is our single instruction exception window. A pending
        // SError is guaranteed to occur at the earliest when we unmask
-- 
2.20.1

_______________________________________________
kvmarm mailing list
kvmarm@lists.cs.columbia.edu
https://lists.cs.columbia.edu/mailman/listinfo/kvmarm

Reply via email to